/**************************************************************************
 *                                                                        *
 *              This code is copyright (c) 1994 & 1995                    *
 *                        Athena Design, Inc.                             *
 *              This code is copyright (c) 1996 & 1997 & 1998             *
 *                       Sundial Systems Corp.                            *
 *                                                                        *
 *                        ALL RIGHTS RESERVED                             *
 *                                                                        *
 **************************************************************************/

/*  This is the external file that must be used be include'd by custom  */
/*  AddIn's that will be linked into Mesa. This file may be included in */
/*  either a C or C++ program.                                          */
/*                                                                      */
/*  All functionality is performed via a call the the                   */
/*  performAddInFunction() function.  The first parameter is an integer */
/*  that denotes the function to be performed.  Additional parameters   */
/*  and the return value is different for each function. Please see the */
/*  "forthcoming" documentation.                                        */


#ifndef _MH_ExternalAddIn_h
#define _MH_ExternalAddIn_h


#ifdef __cplusplus
extern "C"
{
#endif


extern int _System performAddInFunction(int,...);

/* Memory management for some of the functions is handled through       */
/* Mesa's internal memory managment routines.  There are two sets that  */
/* can be used depending on the compiler used.  The first set is        */
/* OptiLinked for use with IBM's C/C++ compilers.  The other set is     */
/* for use with other compilers and languages.  They are _System linked */
/* for portability.  The functions can be used interchangably as the    */
/* system linked ones just call the OptiLink ones internally.           */
#ifndef __MH_MEMORY__
    extern void MFree(void *);
    extern void *MMalloc(int size);
    extern void *MRealloc(void *,int size);

    extern void _System SysMFree(void *);
    extern void *_System SysMRealloc(void *,int size);
    extern void *_System SysMMalloc(int size);
#endif


#ifdef __cplusplus
}
#endif


/* the current list of commands                                         */

#define getMenuWindowMesaAddIn              1
/* HWND performAddInFunction(getMenuWindowMesaAddIn);                   */
/* returns the handle to the application menu or 0 if there is no       */
/* application menu                                                     */


#define registerMenuCallBackMesaAddIn       2
/* int performAddInFunction(registerMenuCallBackMesaAddIn,              */
/*                   module handle, function address)                   */
/* returns 1                                                            */
/* registers a call-back to the menu handling routine.  The module      */
/* handle is the paramter passed to the AddIn when its AddInInit(void *)*/
/* is called.  An AddIn can change the routing called for the menu      */
/* call-back by calling this routing again or may de-register the menu  */
/* callback by passing a NULL as the function address.                  */
/* the function is called in the following format:                      */
/* int _System function(int command,int type) where:                    */
/* command is the menu command (an integer between 27000 and 32767 that */
/* the AddIn associates) with a particular menu command.  An AddIn will */
/* be called with other AddIn's commands and should ignore these        */
/* commands and return a 0 in all circumstances.The type is an integer  */
/* denoting the action to take:                                         */
/* 0 - execute the menu command - return 1 if the command was executed  */
/* 1 - enabled? return a 1 if the AddIn can execute the command (i.e.,  */
/*     the menu should be enabled)                                      */
/* 2 - Open a help dialog for the menu item                             */


#define getActiveModelMesaAddIn             3
/* void *performAddInFunction(getActiveModelMesaAddIn)                  */
/* returns a handle to the currently active model or NULL if there is   */
/* no currently active model                                            */


#define getModelNameMesaAddIn               4
/* char *performAddInFunction(getModelNameMesaAddIn,void *model_handle) */
/* returns a pointer to a string containing the name of the model       */
/* refered to by the handle.  NULL is returned if the model has not     */
/* been named.  The returned string must be free'd from storage with a  */
/* call to MFree()                                                      */


#define setModelNameMesaAddIn               5
/* void performAddInFunction(setModelNameMesaAddIn, void *model_handle, */
/*     const char *new_name,int action)                                 */
/* changes the name of the model to the new name (which should be a     */
/* complete path name)                                                  */
/* if 0 is passed as the action, the windows displaying the model are   */
/* redisplayed with the new model name.  If -1 is passed as the action, */
/* the windows are not redisplayed                                      */


#define openModelMesaAddIn                  6
/* void *performAddInFunction(openModelMesaAddIn,const char *file_name) */
/* If the named model is currently open, it is brought to the front.    */
/* If it is not open, it is loaded off of disk. If it cannot be opened, */
/* NULL is returned, otherwise the model handle is returned             */


#define addressToStringMesaAddIn            7
/* char *performAddInFunction(addressToStringMesaAddIn,                 */
/*         void *model_handle, int row,int col,int layer)               */
/* returns a character string that is the absolute address in row,col,  */
/* layer this string must be MFree'd after it is used                   */


#define rangeToStringMesaAddIn              8
/* char *performAddInFunction(rangeToStringMesaAddIn,                   */
/*         void *model_handle,int row,int col,int layer,int row,        */
/*         int col,int layer)                                           */
/* returns a character string that is the absolute range in row,col,    */
/* layer to row,col,layer this string must be MFree'd after it is used  */


#define selectMesaAddIn                     9
/* void performAddInFunction(selectMesaAddIn, const char *range_name)   */
/* selects the named range on the current window                        */


#define getCurrentAddressMesaAddIn          10
/* int performAddInFunction(getCurrentAddressMesaAddIn, int *row,       */
/*             int *col,int *layer)                                     */
/* returns 1 if there is a valid selection (i.e., an open window with a */
/* spreadsheet layer on top) and thus the row, column, and layer are    */
/* set correctly                                                        */


#define getCurrentRangeMesaAddIn            11
/* int performAddInFunction(getCurrentRangeMesaAddIn, int *row,         */
/*     int *col,int *layer,int *row,int *col,int *layer)                */
/* returns 1 if there is a valid range selection                        */
/* (an open window with a sheet layer showing with a range selected     */
/* (rather than a single cell).  The rectangle of the first part of the */
/* selected range is placed into the correct integers                   */


#define getCellValueMesaAddIn               12
/* void performAddInFunction(getCellValueMesaAddIn,                     */
/*         void *model_handle,MesaAddInValue *aiv)                      */
/* initialize the MesaAddInValue, set the row, column, and layer and    */
/* call this function.  The value of the cell is set and returned.  For */
/* NULL cells or cells with no value, the value is a type of error and  */
/* the error value is 0                                                 */

#define setCellValueMesaAddIn               13
/* int performAddInFunction(setCellValueMesaAddIn,                      */
/*     void *model_handle,MesaAddInValue *aiv,int action)               */
/* sets the given cell to the given value.  Set the value and address   */
/* in the MesaAddInValue, and call this funtion to set the value of a   */
/* given cell.  An error is returned.  0 if successful or non-zero if   */
/* there was an error (the cell address was out of range or the range   */
/* was protected.  A 0 in the action will result in a recalc and a      */
/* redisplay where a -1 will result in neither                          */


#define setExtentsMesaAddIn                 14
/* int performAddInFunction(setExtentsMesaAddIn, void *model_handle,    */
/*         int layer,int maxRow,int maxCol, int action)                 */
/* sets the maximum size of a given layer in the model.                 */
/* The model must have the layer in question as valid for this function */
/* to have any effect.  Set action to 0 to redisplay the worksheets or  */
/* -1 to avoid redisplay events.  Returns 0 if successful, otherwise an */
/* error number                                                         */


#define setBlobMesaAddIn                    15
/* void performAddInFunction(setBlobMesaAddIn, void *model_handle,      */
/*     const char *name,int len,const void *blob_data)                  */
/* sets a named blob (Binary Length Of Bytes) in the model that is      */
/* saved with the model.  The AddIn may save AddIn specific information */
/* with the model via this mechanism                                    */


#define getBlobMesaAddIn                    16
/* int performAddInFunction(getBlobMesaAddIn, void *model_handle,       */
/*     const char *name,int *len,void **blob_data)                      */
/* retrieves a named blob from the model.  Returns 1 if a blob with the */
/* given name was found.  The length of the blob is put in the          */
/* length and the blob_data is set to an allocated block of memory that */
/* contains the blob data.  blob_data must be MFree'd when it is no     */
/* longer needed                                                        */


#define newModelMesaAddIn                   17
/* void *performAddInFunction(newModelMesaAddIn)                        */
/* creates a new model and displays it and returns a handle to the new  */
/* model. Returns 0 if a model cannot be created                        */


#define setColumnSizeMesaAddIn              18
/* int performAddInFunction(setColumnSizeMesaAddIn, void *model_handle, */
/*     int layer, int column,int size, int action)                      */
/* sets the particular column to a particular size in a given layer. If */
/* action is 0, workbooks will be redisplayed, -1 no redisplay.         */
/* Size is in points.                                                   */
/* if the size == -1, reset the size                                    */
/* if size == -2, hide the row or column                                */
/* if size == -3, unhide the row or column                              */
/* if size == -4, set the page break                                    */
/* if size == -5, clear the page break                                  */
/* if size == -6, smart size                                            */
/* returns 0 if successful or error number                              */


#define setRowSizeMesaAddIn                 19
/* int performAddInFunction(setRowSizeMesaAddIn, void *model_handle,    */
/*         int layer, int row,int size, int action)                     */
/* sets the particular row to a particular size in a given layer.  If   */
/* action is 0, workbooks will be redisplayed, -1 no redisplay.         */
/* Size is in points.                                                   */
/* if the size == -1, reset the size                                    */
/* if size == -2, hide the row or column                                */
/* if size == -3, unhide the row or column                              */
/* if size == -4, set the page break                                    */
/* if size == -5, clear the page break                                  */
/* if size == -6, smart size                                            */
/* returns 0 if successful or error number                              */


#define getColumnSizeMesaAddIn              20
/* int performAddInFunction(getColumnSizeMesaAddIn, void *model_handle, */
/*         int layer,int col)                                           */
/* returns the size of the column in points.                            */


#define getRowSizeMesaAddIn                 21
/* int performAddInFunction(getRowSizeMesaAddIn, void *model_handle,int */
/*         layer,int row)                                               */
/* returns the size of the row in points.                               */


#define getExtentsMesaAddIn                 22
/* int performAddInFunction(getExtentsMesaAddIn, void *model_handle,    */
/*     int layer, int *nrows,int *ncols)                                */
/* gets the size of the given layer.  If the layer does not exist in    */
/* the model,0 is returned, otherwise nrows and ncols are set correctly */


#define redisplayMesaAddIn                  23
/* void performAddInFunction(redisplayMesaAddIn, void *model_handle)    */
/* causes all the windows that are watching a given workbook            */
/* to redisplay                                                         */


#define recalcMesaAddIn                     24
/* void performAddInFunction(recalcMesaAddIn, void *model_handle)       */
/* causes the model to recalculate and redisplay.  The recalculation is */
/* done in the background and may be interupted by other user activity  */
/* in the model                                                         */


#define setCellStringMesaAddIn              25
/* int performAddInFunction(setCellStringMesaAddIn,                     */
/*         void *model_handle,const char *string,int layer,int row,     */
/*         int col,int action)                                          */
/* sets a given cell to a string.  The string is parsed (checked for    */
/* being a formula, etc) as if the user typed the string.  action of    */
/* 0 will result in a redisplay and recalc (if recaclc is enabled, -1   */
/* no action.  0 return indicates success, otherwise an error number    */
/* will be returned                                                     */


#define getCellStringMesaAddIn              26
/* char *performAddInFunction(getCellStringMesaAddIn,                   */
/*         void *model_handle, int layer, int row,int col)              */
/* returns the contents of the "formula view" of the cell.  This string */
/* must be MFree'd when it is no longer used                            */


#define getCellOutputMesaAddIn              27
/* char *performAddInFunction(getCellOutputMesaAddIn,                   */
/*         void *model_hanlde,int layer, int row,int col)               */
/* returns the text of what will be displayed in a cell in the exact    */
/* format that it would appear in the cell.  The string must be MFree'd */
/* when it is no longer needed                                          */


#define registerFunctionMesaAddIn           28
/* void performAddInFunction(registerFunctionMesaAddIn,                 */
/*     const char *func_name, const char *fbName,                       */
/*     const char*fbPrototype, const char *fbDescription,               */
/*     const char *fbExample, int *fbMembers, void * callBack)          */
/* func_name - the name of the function, eg: "DOG("                     */
/* fbName - optional (must be NULL if unused) - name as appears in      */
/*         formula bldr: "DOG"                                          */
/* fbPrototype - optional (must be NULL if unused) - prototype in       */
/*         formula builder                                              */
/* fbDescription - optional (must be NULL if unused) - description in   */
/*         formula builder                                              */
/* fbExample - optional (must be NULL if unused) - text example for     */
/*         formula builder                                              */
/* fbMembers - optional (must be NULL if unused) - 0 terminated list of */
/*         function groups for fb                                       */
/* callBack - the address of the function that executes the AddIn       */
/*         function this will be called with the following parameters:  */
/*         void _System function(int numberOfParameters, void *stack,   */
/*                               void *model_handle)                    */
/*         use the popElementMesaAddIn to remove values from the        */
/*         evaluation stack and process them and push the result back   */
/*         on the stack with pushValueMesaAddIn                         */


#define popElementMesaAddIn                 29
/* void performAddInFunction(popElementMesaAddIn,void *stack,           */
/*         void *model_handle, MesaAddInValue *)                        */
/* pops an element off the evaluation stack.  the MesaAddInValue must   */
/* be initialized and free'd after use                                  */


#define pushValueMesaAddIn                  30
/* void performAddInFunction(pushValueMesaAddIn,void *stack,            */
/*         void *model_handle, MesaAddInValue *)                        */
/* pushes an element onto the evaluation stack.                         */


#define registerDoubleClickMesaAddIn        31
/* int performAddInFunction(registerDoubleClickMesaAddIn,               */
/*          void * module_handle,void *function_address)                */
/* registers the call-back for double-clicking on a cell                */
/* if the user double-clicks on a cell in a sheet view, the AddIn is    */
/* called in the following way:                                         */
/* void _System function(void *model_handle,intlayer,                   */
/*             int row, int column)                                     */


#define registerAboutToSaveMesaAddIn        32
/* int performAddInFunction(registerAboutToSaveMesaAddIn,               */
/*         void *module_handle,void *function_address)                  */
/* registers the call-back for the pre-save operation just before a     */
/* model is saved out to disk, this call-back is called.  This is an    */
/* opportunity for the AddIn to place information with the model or to  */
/* verify that the model has the correct contents. If the AddIn returns */
/* '0' the model is saved (assuming that other addin's return 0),       */
/* otherwise, the save operation is aborted and the                     */
/* saveBlockedByAddInError error is returned                            */
/* int _System function(void *model_handle)                             */


#define registerOpenedMesaAddIn             33
/* int performAddInFunction(registerOpenedMesaAddIn,                    */
/*         void *module_handle,void *function_address)                  */
/* registers the call-back for the post-open operation this call-back   */
/* is called each time a model is opened from disk.  This is an         */
/* oportunity for an AddIn to check to see if a Model could use the     */
/* AddIn's services for example, by checking the BLOB info for a        */
/* registered blob                                                      */
/* void _System function(void *model_handle)                            */


#define registerAboutToCloseMesaAddIn       34
/* int performAddInFunction(registerAboutToCloseMesaAddIn,              */
/*     void *module_handle,void *function_address)                      */
/* registers the call-back for the pre-close operation this call-back   */
/* is called just before the last window to a model is closed.  This    */
/* in an opportunity for the AddIn to block the close operation or to   */
/* place data in the model before it is closed.  Return 1 to block the  */
/* close operation                                                      */
/* int _System function(void *model_handle)                             */


#define registerDataEnteredMesaAddIn        35
/* int performAddInFunction(registerDataEnteredMesaAddIn,               */
/*         void *module_handle,void *function_address)                  */
/* registers the call-back for the pre-enter data operation this        */
/* call-back is called just before data is entered into a cell. This    */
/* gives the addin the opportunity to do data validation and optionally */
/* block the entry of data into the cell.  return 1 to block data entry */
/* or 0 to allow it.  A entryBlockedByAddInError is returned if the     */
/* data addin blocks the data                                           */
/* int _System function(void * model_handle,int layer,int row,          */
/*         int column,const char *data)                                 */


#define registerPMCallBackMesaAddIn         36
/* int performAddInFunction(registerPMCallBackMesaAddIn, module handle, */
/*         function address)                                            */
/* returns 1                                                            */
/* registers a call-back to the PM Message handling routine. The module */
/* handle is the paramter passed to the AddIn when its                  */
/* AddInInit(void *) is called.  An AddIn can change the routing        */
/* called for the PM call-back by calling this routing again or may     */
/* de-register the PM callback by passing a NULL as the function        */
/* address. The function is called in the following format:             */
/* MRESULT _System function(HWND window,ULONG msg,MPARAM mp1,MPARAM mp2)*/
/* Messages greater than WM_USER + 1000 get passed to all addins. It is */
/* the addin's job to choose a unique PM Message number (sorry guys, no */
/* tips).  The MRESULTS are or'ed together so if you don't recognize    */
/* the message, return 0                                                */


#define getActiveWindowMesaAddIn            37
/* HWND performAddInFunction(getActiveWindowMesaAddIn)                  */
/* returns the HWND for the currently active spreadsheet window or 0 if */
/* there is no window                                                   */


#define getApplicationWindowMesaAddIn       38
/* HWND performAddInFunction(getApplicationWindowMesaAddIn)             */
/* returns the HWND for the application window.  This function can      */
/* return 0 if there is no application window (i.e., the addin is       */
/* running in a custom application via MOLI)                            */


#define initValueMesaAddIn                  39
/* void performAddInFunction(initValueMesaAddIn, MesaAddInValue *)      */
/* initializes the data structure to default values.  Must be done      */
/* before the data structure is used in any calls                       */


#define freeValueMesaAddIn                  40
/* void performAddInFunction(freeValueMesaAddIn, MesaAddInValue *)      */
/* releases any storage used by the MesaAddInValue.  This must be       */
/* called before the MesaAddInValue goes out of scope                   */


#define getNumberOfLayersMesaAddIn          41
/* int performAddInFunction(getNumberOfLayersMesaAddIn,                 */
/*         void *model_handle)                                          */
/* returns the number of layers in the model                            */


#define addALayerMesaAddIn                  42
/* void performAddInFunction(addALayerMesaAddIn,void *model_handle,     */
/*             int action)                                              */
/* adds a layer to the model.  if the "action" parameter is 0,          */
/* redisplay takes place                                                */
/* if the action parameter is -1, no redisplay takes place              */


#define saveModelAddIn                      43
/* int performAddInFunction(saveModelAddIn, void *model_handle)         */
/* call to save the model.  Returns 0 if successful or error number     */


#define stringToAddressMesaAddIn            44
/* int performAddInFunction(saveModelAddIn, void *model_handle,char     */
/* *str,int *r, int *c, int *l)                                         */
/* returns 1 if sussessful                                              */


#define stringToRangeMesaAddIn              45
/* int performAddInFunction(saveModelAddIn, void *model_handle,         */
/*     char *str,int item, int *r1, int *c1, int *l1,int *r2,           */
/*     int *c2, int*l2)                                                 */
/* returns 0 if unsussessful, the total item count if it is             */
/* (multi-ranges)                                                       */


#define getVersionNumberMesaAddIn           46
/* int performAddInFunction(getVersionNumberMesaAddin)                  */
/* returns the Version number of Mesa.  This is the build number found  */
/* on the About panel of the Mesa 2 Application.                        */


#define cycleCalcIndicatorMesaAddIn         47
/* void performAddInFunction(cycleCalcIndicatorMesaAddIn)               */
/* cycles the little abacus in the lower left hand corner of the        */
/* screen. Use this to indicate that you are doing a long operation on  */
/* a background thread.                                                 */


#define setProgressIndicatorMesaAddIn       48
/* void performAddInFunction(setProgressIndicatorMesaAddIn,int)         */
/* sets the percent complete (0-100) on the status area of the status   */
/* bar. Start by setting it to zero, then incrementing it until done.   */
/* When done, set it to 100 so it knows that everything is done and it  */
/* can start displaying the tip text.                                   */


#define isEditingMesaAddIn                  49
/* int performAddInFunction(isEditingMesaAddIn)                         */
/* Returns one of the following values:                                 */
/*    2  if the user is editing a formula                               */
/*    1  if the user is editing something other than a formula (a       */
/*       string/date/etc..)                                             */
/*    0  if the user is not in  edit mode                               */


#define addStringToHistoryMesaAddIn         50
/* void performAddInFunction(addStringToHistoryMesaAddIn, char * str)   */
/* Can be used to add strings to the script recorder.  If you support a */
/* REXX function for actions you do on the model, you can record the    */
/* action here                                                          */


#define setRangeToFontMesaAddIn             51
/* int performAddInFunction(setRangeToFontMesaAddIn,                    */
/*     void *model_handle, char *font,int r, int c, int l, int r2,      */
/*     int c2,int l2,int action)                                        */
/*  The font string is in system format.  Example: "12.Helvetica"       */
/* Returns 0 if successful                                              */


#define setRangeFGColorMesaAddIn            52
/* int performAddInFunction(setRangeFGColorMesaAddIn,                   */
/*     void *model_handle, int color,int r, int c, int l, int r2,       */
/*     int c2, int l2,int action)                                       */
/* Returns 0 if successful                                              */


#define setRangeBGColorMesaAddIn            53
/* int performAddInFunction(setRangeBGColorMesaAddIn,                   */
/*         void *model_handle, int color,int r, int c, int l, int r2,   */
/*         int c2, int l2,int action)                                   */
/* Color is an RGB value                                                */
/* Returns 0 if successful                                              */


#define getFontMesaAddIn                    54
/* char *performAddInFunction(getBGColorMesaAddIn, void *model_handle,  */
/*     int r, int c, int l)                                             */
/* returns the font string of the cell.  You must MFree the returned    */
/* string                                                               */


#define getBGColorMesaAddIn                 55
/* int performAddInFunction(getBGColorMesaAddIn, void *model_handle,    */
/*     int r, int c, int l)                                             */
/* returns the back ground color of the cell                            */


#define getFGColorMesaAddIn                 56
/* int performAddInFunction(getFGColorMesaAddIn, void *model_handle,    */
/*         int r, int c, int l)                                         */
/* returns the for ground color of the cell                             */


#define setRangeBorderMesaAddIn             57
/* int performAddInFunction(setRangeBorderMesaAddIn,                    */
/*     void *model_handle,int code,int thick,int color,int r, int c,    */
/*     int l, int r2, int c2, int l2,int action)                        */
/* The code is one of:             The thickness is one of:             */
/*      0 : Left                            0 : None                    */
/*      1 : Right                           1 : Thin                    */
/*      2 : Top                             2 : Thick                   */
/*      3 : Bottom                          3 : Very Thick              */
/* Color is an RGB value                                                */
/* Returns 0 if successful                                              */


#define registerRecalcDoneMesaAddIn         58
/* void performAddInFunction(registerRecalcDoneMesaAddIn,               */
/*     void *model_handle, void *function_address)                      */
/* registers the call-back for the end of recalc                        */
/* operation.  You can use this to get notification as to when a recalc */
/* is finished. This function is called on the Model thread,(the        */
/* background recalc thread) not the                                    */
/* UI thread, because of this, you cannot do any read type operations   */
/* while processing this call. Otherwise, the application will lock up. */
/* int function(void *model_handle)                                     */


#define setRangeAlignmentMesaAddIn          59
/* int performAddInFunction(setRangeBorderMesaAddIn,                    */
/*         void *model_handle,int alignment,int r, int c, int l,        */
/*         int r2, int c2, int l2,int action)                           */
/* The code is alignment of the following:                              */
/*      0 : General (smart)                                             */
/*      1 : Left                                                        */
/*      2 : Center                                                      */
/*      3 : Right                                                       */
/*      4 : Top                                                         */
/*      5 : Vert. Center                                                */
/*      6 : Bottom                                                      */
/*      7 : Wrapped                                                     */
/*      8 : Unwrapped                                                   */


#define setRangeFormatMesaAddIn             60
/* int performAddInFunction(setRangeFormatMesaAddIn,void *model_handle, */
/*         int format,int prec,int r, int c, int l, int r2, int c2,     */
/*         int l2,int action)                                           */
/* The format is on fo the following:                                   */
/*      0  : General                11 : Date (MM/DD)       22 : Hidden */
/*      1  : Currency               12 : Thousands                      */
/*      2  : Percent                13 : Text                           */
/*      3  : Scientific             14 : Currency1                      */
/*      4  : Fixed                  15 : Currency2                      */
/*      5  : Date (DD-MON-YY)       16 : Currency3                      */
/*      6  : Date (DD.MM.YY)        17 : Time (HMS12)                   */
/*      7  : Date (MON-DD-YY)       18 : Time (HM12)                    */
/*      8  : Date (DD-MON)          19 : Time (HMS24)                   */
/*      9  : Date (MON-DD)          20 : Time (HM24)                    */
/*      10 : Date (MM/DD/YY)        21 : Paren                          */
/* The precision can be set to -1 to use the default precision for the  */
/* desired format.  Dome formats ignore the precision altogether.       */


#define setLayerNameMesaAddIn               61
/* int performAddInFuntion(setLayerNameMesaAddIn,void *model_handle,    */
/*          int layer, const char *name,int action)                     */
/* Sets the name of the specied layer to the given name.  If action is  */
/* -1, then the screen is not redrawn to reflect the change.            */


#define setMultiValueMesaAddIn              62
/* int performAddInFunction(setMultiValueMesaAddIn,                     */
/*     void *model_handle,MesaAddInValue *aiv,int num,int action)       */
/* Sets a bunch of AddInValues into the model.  Because only one        */
/* message is sent to the Model, this can be much faster than using     */
/* setCellValueMesaAddIn if you are setting a whole bunch of values     */
/* into a model.  Num is the number of AddInValues in the array pointed */
/* to by aiv.  See setCellValueMesaAddIn for other notes.               */

#define createGraphMesaAddIn                63
/* char *performAddInFunction(createGraphMesaAddIn,void *model_handle,  */
/*    int type, int r1,int c1, int l1,int r2,int c2, int l2,            */
/*    int horsize, int versize, int horoffset,int veroffset,int action) */
/* Type is one of the following:                                        */
/*    Column Chart = 0                                                  */
/*    Bar Chart = 1                                                     */
/*    Stacked Column Chart= 2                                           */
/*    Stacked Bar Chart = 3                                             */
/*    Bar3 Chart = 4                                                    */
/*    Pyr3 Chart = 5                                                    */
/*    Area Chart = 6                                                    */
/*    Pie Chart = 7                                                     */
/*    Line Chart = 8                                                    */
/*    Area3 Chart = 9                                                   */
/*    HiLow Chart = 10                                                  */
/*    Scatter Chart = 11                                                */
/*    Ribbon3 Chart = 12                                                */
/*    Candle Chart = 13                                                 */
/*    BarLine Chart = 14                                                */
/* The size values are in points.  The offsets are the offset in points */
/* For the top left hand corner of the graph from the upper left cell   */
/* in the range.                                                        */
/* Returns the name of the newly created frame (MFree the string)       */

#define setGraphTitleMesaAddIn              64
/* void performAddInFunction(setGraphTitleMesaAddIn, void *model_handle,*/
/*    char *name, int type, char *title, int action)                    */
/* Type is one of the following:                                        */
/*    0 = Main title                                                    */
/*    1 = Sub title                                                     */
/*    2 = X-Axis                                                        */
/*    3 = Y-Axis                                                        */

#define moveGraphMesaAddIn                  65
/* void performAddInFunction(moveGraphMesaAddIn, void *model_handle,    */
/*    char *frame, int row, int col, int layer,                         */
/*    int horsize, int versize, int horoffset,int veroffset,int action) */
/* Moves a frame to to a new location.  The row, col, and layer values  */
/* are the cell to act as the base cell or anchor point for the frame.  */
/* Setting the layer to a layer other than where the data is located    */
/* will move the graph onto a different layer.                          */

#define setGraphHeadingLocationMesaAddIn    66
/* void performAddInFunction(setGraphHeadingLocationMesaAddIn,          */
/*    void * model_handle, char *frame, int flag, int action);          */
/* Can be used to set the "Heading on Axis" flags for the given frame.  */
/* The flag is on of the following:                                     */
/*    0 - no headings in data                                           */
/*    1 - headings from first row                                       */
/*    2 - headings from first column                                    */
/*    3 - headings from both the first row and first column             */

#define setLegendLocationMesaAddIn          67
/* void performAddInFunction(setLegendLocationMesaAddIn,                */
/*    void * model_handle, char *frame, int flag, int action);          */
/* Sets the location of the legnd for the given frame.                  */
/* The flag is one of the following:                                    */
/*    0 - No legend                                                     */
/*    1 - Left                                                          */
/*    2 - Right                                                         */
/*    3 - Top                                                           */
/*    4 - Bottom                                                        */

#define setGraphOrientationMesaAddIn        68
/* void performAddInFunction(setGraphOrientationMesaAddIn,              */
/*    void * model_handle, char *frame, int flag, int action);          */
/* Sets whether the data orientation should be grouped by row or        */
/* by column.                                                           */
/* Flag is 0 for orient by row, 1 for by column.                        */

#define setGraphLinesMesaAddIn              69
/* void performAddInFunction(setGraphLinesMesaAddIn,                    */
/*    void * model_handle, char *frame, int flag, int action);          */
/* Sets the line type for the graph.  Flag is:                          */
/*    0 - don't show lines or points                                    */
/*    1 - show lines                                                    */
/*    2 - show points                                                   */
/*    3 - show lines and points                                         */

#define deleteFrameMesaAddIn                70
/* void performAddInFunction(setGraphLinesMesaAddIn,                    */
/*    void * model_handle, char *frame, int action);                    */
/* Deletes the frame.                                                   */

#define copyMesaAddIn                       71
/* void performAddInFunction(copyMesaAddIn,void * model_handle,         */
/*    int flags,int r1, int c1, int l1, int r2, int c2, int l2)         */
/* Copies the range to the system clipboard.  Flags is used to dermine  */
/* formats are copied to the clipboard.  It can be any combination of:  */
/*      1 - Plain Text (tab separated)                                  */
/*      2 - Rich Text Format                                            */
/*      4 - Mesa Format                                                 */
/*      8 - Bitmap                                                      */
/*     16 - MetaFile                                                    */
/*     32 - DDE Link information                                        */
/* Performance can be increased by only copying the desired formats.    */

#define pasteMesaAddIn                      72
/* int performAddInFunction(pasteMesaAddIn,void * model_handle,         */
/*    int flags,int r1, int c1, int l1, int r2, int c2, int l2,         */
/*    int action)                                                       */
/* Pastes data from the systems clipboard into the desired range.       */
/* Flags can be one of the following:                                   */
/*      0 - Normal Paste                                                */
/*      1 - Values Paste                                                */
/*      2 - Format Paste                                                */
/*      3 - Transpose Paste                                             */
/*      4 - Insert Down Paste                                           */
/*      5 - Insert Right Paste                                          */
/*      6 - Move Paste                                                  */
/*      7 - Link Paste                                                  */

#define copyPasteMesaAddIn                  73
/* int performAddInFunction(copyPasteMesaAddIn,void * model_handle,     */
/*    int flags,int r1, int c1, int l1, int r2, int c2, int l2,         */
/*    int r3, int c3, int l3, int r4, int c4, int l4, int action)       */
/* Performs the copy and paste in one operation.  This does NOT modify  */
/* the system clipboard, thus the operation can be completed much       */
/* faster when compared to the two above operations that do modify the  */
/* clipboard.                                                           */
/* flags can be any of the values defined under the pasteMesaAddIn      */

#define importTextMesaAddIn                 74
/* int performAddInFunction(importTextMesaAddIn,void * model_handle,    */
/*    int r, int c, int l,  const char *txt, const char *delims,        */
/*    int flags, int action)                                            */
/* Performs a text import using the text passed in the txt parameter.   */
/* The import starts at the cell specified by r, c, and l.  The         */
/* column delimiters to use during the export is specified in the       */
/* delims parameter.  flags can be any combination of the following:    */
/*      importFlagTreatMultiDelimAsOne  0x0001                          */
/*      importFlagTextQulaifierQuote    0x0002                          */
/*      importFlagTextQulaifierDblQuote 0x0004                          */
/*      dontExpandSheetImportFlag       0x1000                          */
/*      dontExpandRowsImportFlag        0x2000                          */
/*      dontExpandColsImportFlag        0x4000                          */

#define setGraphPropertyMesaAddIn           75
/* int performAddInFunction(setGraphPropertyMesaAddIn,                  */
/*      void * model_handle, const char *framename,                     */
/*      const char *property, const char * value,int action)            */
/* Sets the specified property of a graph to the given value.   For a   */
/* list of property/value pairs, see the online help for the            */
/* SETGRAPHPROPERTY MScript function.                                   */

#define setPrintPropertyMesaAddIn           76
/* int performAddInFunction(setPrintPropertyMesaAddIn,                  */
/*      void * model_handle, int layernum,                              */
/*      const char *property, const char * value)                       */
/* sets the specified print property for the given layer.               */
/*  See the SETPRINTSETTING MScript function for a list of properties   */

#define instantiateRangeMesaAddIn           77
/* int performaAddInFunction(instantiateRangeMesaAddIn,                */
/*      void * model_handle, int r1, int c1, int l1,                    */
/*      int r2, int c2, int l2)                                         */
/* Makes sure that all the cells in the given range are created and     */
/* reside in memory.  This can speed up large block operations.         */

#define setStatusMessageMesaAddIn           78
/* void performAddInFunction(setStatusMessageMesaAddIn,const char *)    */



/* AddIn functions from 79 through 102 require Mesa 2 build 348 or      */
/* higher.  Make sure you check that before using them.                 */

#define registerInitMenuMesaAddIn           79
/* int performAddInFunction(registerInitMenuMesaAddIn,                  */
/*     void *module_handle,void *function_address)                      */
/* This register a function that is called whenever the menus are       */
/* reloaded/recreated.  This is a good point to add menu items, remove  */
/* others, etc....  If a user changes the menu structure, this will be  */
/* recalled so you can add your items.                                  */
/* void _System function(int menu, HWND hwndMenu)                       */
/* menu will be one of:                                                 */
/*    0 - main application menu                                         */
/*    1 - range menu                                                    */
/*    2 - row heading                                                   */
/*    3 - col heading                                                   */
/*    4 - graphic                                                       */

#define setRangeProtectionMesaAddIn         80
/* int performAddInFunction(setRangeProtectionMesaAddIn,                */
/*         void *model_handle, int protect,int r, int c, int l, int r2, */
/*         int c2, int l2,int action)                                   */
/*  sets the protection attibute on each cell in the range              */

#define setLayerProtectionMesaAddIn         81
/* int performAddInFunction(setLayerProtectionMesaAddIn,                */
/*     void *model_handle,int layer, int protect, int action)           */
/* sets the protection level for the given layer.                       */

#define setLayerGridMesaAddIn               82
/* int performAddInFunction(setLayerGridMesaAddIn,                      */
/*     void *model_handle,int layer, int grid, int action)              */
/* Sets the visibility of the grid lines for the given layer.           */

#define getLayerNameMesaAddIn               83
/* char * performAddInFunction(getLayerNameMesaAddIn,                   */
/*     void *model_handle,int layer)                                    */
/* Returns the name of the layer.  You must MFree the name when done.   */

#define getLayerNumberMesaAddIn             84
/* int performAddInFunction(getLayerNameMesaAddIn,                      */
/*     void *model_handle,const char *layer)                            */
/* Finds the layer number for the given name.  Will return -1 if no     */
/* such layer exists.                                                   */

#define printMesaAddIn                      85
/* int performAddInFunction(printMesaAddIn,                             */
/*     void *model_handle, const char *what, int firstP, int lastP)     */
/*  what is either a range as a text string, a graph name, a layer      */
/* name or NULL to print the whole workbook. firstP and lastP are the   */
/* fist and last pages in the range of pages to print.  If you want to  */
/* do the whole print job, use firstP of 1 and lastP of something very  */
/* large like 99999,                                                    */
    

#define setPrinterMesaAddIn                 86
/* int performAddInFunction(setPrinterMesaAddIn,                        */
/*     void *model_handle, const char *pname)                           */
/* sets the name of the printer to use as during print jobs.            */

#define setCellCommentMesaAddIn             87
/* int performAddInFunction(setCellCommentMesaAddIn,                    */
/*     void *model_handle,int r, int c, int l, const char * comment,    */
/*     int action)                                                      */
/* Sets the cell comment to the given text.                             */

#define getCellCommentMesaAddIn             88
/* char * performAddInFunction(getCellCommentMesaAddIn,                 */
/*     void *model_handle,int r, int c, int l)                          */
/* returns the cell comment for the given cell or NULL if no comment    */
/* is currently in the cell.  You must MFree the returned value.        */

#define autoFormulaMesaAddIn                89
/* int performAddInFunction(autoFormulaMesaAddIn,                       */
/*         void *model_handle, int type,int r, int c, int l, int r2,    */
/*         int c2, int l2,int action)                                   */
/*  type is one of:                                                     */
/*          0 - sum                                                     */
/*          1 - average                                                 */
/*          2 - stddev                                                  */
/*          3 - var                                                     */
/*          4 - min                                                     */
/*          5 - max                                                     */
/*          6 - product                                                 */
/*          7 - sumsq                                                   */

#define fillRangeMesaAddIn                  90
/* int performAddInFunction(fillRangeMesaAddIn,                         */
/*         void *model_handle, int type,int r, int c, int l, int r2,    */
/*         int c2, int l2,int action)                                   */
/*  type is one of:                                                     */
/*          0 - Smart right                                             */
/*          1 - Smart down                                              */
/*          2 - right                                                   */
/*          3 - down                                                    */
/*          4 - matrix                                                  */

#define convertRangeMesaAddIn               91
/* int performAddInFunction(convertRangeMesaAddIn,                      */
/*         void *model_handle, int type,int r, int c, int l, int r2,    */
/*         int c2, int l2,int action)                                   */
/*  type is one of:                                                     */
/*         0 - formula to values                                        */
/*         1 - strings to numbers                                       */
/*         2 - numbers to strings                                       */
/*         3 - to upper case                                            */
/*         4 - to lower case                                            */
/*         5 - to proper case                                           */



#define exportBitmapMesaAddIn               92
#define exportMetafileMesaAddIn             93
#define exportHTMLMesaAddIn                 94
#define exportRTFMesaAddIn                  95
/* int performAddInFunction(exportTYPEMesaAddIn,                  */
/*     void *model_handle, const char *filename,                  */
/*     const char *what)                                          */
/* what can be a range string, range label, or graph name         */

#define exportTextMesaAddIn                 96
/* int performAddInFunction(exportTextMesaAddIn,                  */
/*     void *model_handle, const char *filename,                  */
/*     const char *what, const char *delim, int textSep)          */
/* textSep:  0 - none                                             */
/*           1 - single quote                                     */
/*           2 - double quote                                     */
/* what can be a range string, range label                        */

#define deleteRowsMesaAddIn                 97
/* int performAddInFunction(deleteRowsMesaAddIn,                  */
/*     void *model_handle,int layer, int firstRow, int lastRow,   */
/*     int action)                                                */
/* deletes the given range of rows                                */

#define deleteColsMesaAddIn                 98
/* int performAddInFunction(deleteColsMesaAddIn,                  */
/*     void *model_handle,int layer, int firstCol, int lastCol,   */
/*     int action)                                                */
/* deletes the given range of columns                             */

#define insertRowsMesaAddIn                 99
/* int performAddInFunction(insertRowsMesaAddIn,                  */
/*     void *model_handle,int layer, int firstRow, int lastRow,   */
/*     int action)                                                */
/* inserts rows before the given rows.                            */

#define insertColsMesaAddIn                 100
/* int performAddInFunction(insertColsMesaAddIn,                  */
/*     void *model_handle,int layer, int firstCol, int lastCol,   */
/*     int action)                                                */
/* inserts columns before the given columns.                      */

#define insertLayerMesaAddIn                101
/* int performAddInFunction(insertLayerMesaAddIn,                 */
/*     void *model_handle,int layer, int action)                  */
/* inserts a layer before the given layer.                        */


#define deleteLayerMesaAddIn                102
/* int performAddInFunction(deleteLayerMesaAddIn,                 */
/*     void *model_handle,int layer, int action)                  */
/* deletes the given layer                                        */


#ifdef __cplusplus
extern "C" 
{ 
#endif

typedef struct _MesaAddInValue {
    int row;
    unsigned short column;
    unsigned short layer;
    int type;
    double number;
    char *string;
    int error;
    unsigned short rows;
    unsigned short cols;
    struct _MesaAddInValue * array;
} MesaAddInValue;

#ifdef __cplusplus
}
#endif

#define errorValueMesaAddInType 0
#define stringValueMesaAddInType 1
#define numberValueMesaAddInType 2
#define arrayValueMesaAddInType 3

// ifndef _MH_ExternalAddIn_h
#endif
